Imports System.Web.Mvc
Imports System.Web.Routing
Imports BankDemo.Cqrs.CommandStack.Sagas
Imports BankDemo.Cqrs.QueryStack.Denormalizers
Imports Memento.Messaging
Imports Memento.Messaging.Postie
Imports Memento.Messaging.Postie.Unity
Imports Memento.Persistence
Imports Memento.Persistence.EmbeddedRavenDB
Imports Microsoft.Practices.Unity
Imports Raven.Client.Embedded
Imports Raven.Database.Server

Namespace BankDemo
	Public Class MvcApplication
		Inherits HttpApplication

		Private Shared privateBus As IBus
		Public Shared Property Bus As IBus
			Get
				Return privateBus
			End Get
			Private Set(ByVal value As IBus)
				privateBus = value
			End Set
		End Property

		Private Shared privateAggregateRepository As IRepository
		Public Shared Property AggregateRepository As IRepository
			Get
				Return privateAggregateRepository
			End Get
			Private Set(ByVal value As IRepository)
				privateAggregateRepository = value
			End Set
		End Property

		Protected Sub Application_Start()
			AreaRegistration.RegisterAllAreas()
			RouteConfig.RegisterRoutes(RouteTable.Routes)

			' Initialize the event store (RAVENDB)
			NonAdminHttp.EnsureCanListenToWhenInNonAdminContext(8080)
			Dim documentStore = New EmbeddableDocumentStore With {.ConnectionStringName = "EventStore", .UseEmbeddedHttpServer = True}
			documentStore.Configuration.Port = 8080
			documentStore.Initialize()

			' Configure the FX
			Dim container = MementoStartup.UnityConfig(Of InMemoryBus, EmbeddedRavenDbEventStore, EmbeddableDocumentStore)(documentStore)

			' Save global references to the FX core elements
			Bus = container.Resolve(Of IBus)()
			AggregateRepository = container.Resolve(Of IRepository)()

			' Add sagas and handlers to the bus
			Bus.RegisterSaga(Of AccountRequestSaga)()
			Bus.RegisterHandler(Of AccountRequestDenormalizer)()
		End Sub
	End Class
End Namespace
